#pragma once
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

typedef unsigned char BYTE;

#define SAFEDELETE(x) \
	if (x) delete [] x;

class Alphabet {
public:
	void Init();
	int GetOffset(BYTE b);
	void MoveToFront(int offset);
	BYTE GetCharacter(int offset);

private:
	BYTE alphabet[256];
	int alphabetSize;
};

class Table {
public:
	Table();
	void Init(BYTE *tempSequence, int length);
	BYTE GetByte(int tableIndex, int offset);
	void PrintTable();
	void SortTable();
	void CreateLastSequence();
	void CreateFirstSequence();
	int GetOriginalSequenceIndex();

private:
	static int SortCompare(const void* a, const void* b);

	int sequenceLength;
	BYTE* sequence;
	int* sequenceTable;
	int sequenceTableIndex;

public:
	BYTE* lastSequence;
	BYTE* firstSequence;
};

extern Table table;

// Currently not used, but I may need it if I do a bucket sort/suffix array sort
template <class T>
class Vector {
public:
	Vector() {
		capacity = 256;
		size = 0;
		v = new T[capacity];
	}

	Vector(int capacity) {
		this->capacity = capacity;
		size = 0;
		v = new T[capacity];
	}

	~Vector() {
		delete [] v;
	}

	void Push(T a) {
		if (size == capacity)
			Resize();
		v[size++] = a;
	}

	void Resize() {
		capacity *= 2;
		T *v2 = new T[capacity];
		memcpy(v2, v, size*sizeof(T));
		delete [] v;
		v = v2;
	}

public:
	int size;
	T* v;

protected:
	int capacity;
};