#include "BW.h"

Table::Table() {
	sequence = NULL;
	sequenceTable = NULL;
	lastSequence = NULL;
	firstSequence = NULL;
}

void Table::Init(BYTE *tempSequence, int length) {
	SAFEDELETE(sequence);
	SAFEDELETE(sequenceTable);
	SAFEDELETE(lastSequence);
	SAFEDELETE(firstSequence);

	sequence = new BYTE[length];
	sequenceTable =  new int[length];
	lastSequence = new BYTE[length];
	firstSequence = new BYTE[length];
	memcpy(sequence, tempSequence, length);
	
	for (int i = 0; i < length; i++) {
		sequenceTable[i] = i;
	}
	sequenceLength = length;
	SortTable();
	CreateLastSequence();
	CreateFirstSequence();
}

BYTE Table::GetByte(int tableIndex, int offset) {
	int toffset = sequenceTable[tableIndex] + offset;
	if (toffset >= sequenceLength)
		toffset -= sequenceLength;
	return sequence[toffset];
}

void Table::PrintTable() {
	printf("Table:\n");
	for (int i = 0; i < sequenceLength; i++) {
		for (int j = 0; j < sequenceLength; j++) {
			BYTE b = GetByte(i, j);
			if (b < 32)
				printf(".");
			else
				printf("%c", b);
		}
		printf("\n");
	}
}

int Table::SortCompare(const void* a, const void* b) {
	int aIndex = (int)((int *)a - table.sequenceTable);
	int bIndex = (int)((int *)b - table.sequenceTable);

	for (int i = 0; i < table.sequenceLength; i++) {
		int diff = table.GetByte(aIndex, i) - table.GetByte(bIndex, i);

		if (diff)
			return diff;
	}
	return 0;
}

void Table::SortTable() {
	// I think this will be O(n^2 log n)
	qsort(sequenceTable, sequenceLength, sizeof(int), SortCompare);
}

void Table::CreateLastSequence() {
	for (int i = 0; i < sequenceLength; i++) {
		lastSequence[i] = GetByte(i, sequenceLength-1);
	}
}

void Table::CreateFirstSequence() {
	for (int i = 0; i < sequenceLength; i++) {
		firstSequence[i] = GetByte(i, 0);
	}
}

int Table::GetOriginalSequenceIndex() {
	for (int i = 0; i < sequenceLength; i++) {
		if (sequenceTable[i] == 0)
			return i;
	}

	return -1;
}