#include <Windows.h>
#include "BW.h"

Table table;

void Encode(const char *inFile, const char *outFile, unsigned int maxSequenceLength) {
	FILE *in = fopen(inFile, "rb");
	if (!in) {
		printf("Unable to open the file\n");
		return;
	}
	FILE *out = fopen(outFile, "wb");
	if (!out) {
		printf("Unable to open the file\n");
		return;
	}

	Alphabet alphabet;

	fwrite(&maxSequenceLength, sizeof(maxSequenceLength), 1, out);

	alphabet.Init();

	BYTE* tempSequence = new BYTE[maxSequenceLength];
	int len;
	while ((len = (int)fread(tempSequence, sizeof(BYTE), maxSequenceLength, in)) > 0) {
		table.Init(tempSequence, len);

		unsigned int index = table.GetOriginalSequenceIndex();
		fwrite(&index, sizeof(index), 1, out);

		for (int i = 0; i < len; i++) {
			BYTE offset = alphabet.GetOffset(table.lastSequence[i]);
			alphabet.MoveToFront(offset);
			fwrite(&offset, sizeof(BYTE), 1, out);
		}
		printf(".");
	}
	delete [] tempSequence;

	fclose(in);
	fclose(out);
}

BYTE* lastSequence;

int alphabetizeSort(const void* a, const void *b) {
	int aIdx = *(int *)a;
	int bIdx = *(int *)b;
	BYTE A = lastSequence[aIdx];
	BYTE B = lastSequence[bIdx];

	if (A == B) {	// We need to make sure the first letter stays first when sorted
		return aIdx - bIdx;
	}
	return A - B;
}

void Decode(const char *inFile, const char *outFile) {
	FILE *in = fopen(inFile, "rb");
	if (!in) {
		printf("Unable to open the file\n");
		return;
	}
	FILE *out = fopen(outFile, "wb");
	if (!out) {
		printf("Unable to open the file\n");
		return;
	}

	Alphabet alphabet;

	// HEADER
	unsigned int maxSequenceLength;
	fread(&maxSequenceLength, sizeof(maxSequenceLength), 1, in);

	alphabet.Init();

	BYTE* tempData = new BYTE[maxSequenceLength];
	int* referenceSequence = new int[maxSequenceLength];
	lastSequence = new BYTE[maxSequenceLength];
	unsigned int index;
	while (fread(&index, sizeof(index), 1, in) == 1) {
		int len;
		len = (int)fread(tempData, 1, maxSequenceLength, in);

		if (len > 0) {
			// Undo the MTF stuff
			for (int i = 0; i < len; i++) {
				lastSequence[i] = alphabet.GetCharacter(tempData[i]);
				referenceSequence[i] = i;
				alphabet.MoveToFront(tempData[i]);
			}
			// Sort referenceSequence
			qsort(referenceSequence, len, sizeof(int), alphabetizeSort);

			for (int i = 0; i < len; i++) {
				index = referenceSequence[index];
				fwrite(&lastSequence[index], sizeof(BYTE), 1, out);
			}
			printf(".");
		}
	}
	delete [] referenceSequence;
	delete [] lastSequence;
	delete [] tempData;

	fclose(in);
	fclose(out);
}

void main(int argc, char** argv)
{
	if (argc != 4 && argc != 5) {
		printf("Encode usage: bw -e [block size in bytes] [in file] [out file]\n");
		printf("\tExample: bw -e 2048 aesop-4copies.txt aesop-4copies.txt.bw\n");
		printf("Decode usage: bw -d [in file] [out file]\n");
		printf("\tExample: bw -d aesop-4copies.txt.bw aesop-4copies.txt\n");
		printf("\n\nWarning: Block sizes beyond 200,000 may cause a significant slowdown when encoding.");
		return;
	}

	if (_stricmp(argv[1], "-e") == 0) {
		unsigned int start = timeGetTime();
		Encode(argv[3], argv[4], atoi(argv[2]));
		printf("Compressed in %.2f seconds\n", (timeGetTime() - start) * 0.001f);
	}
	else if (_stricmp(argv[1], "-d") == 0) {
		Decode(argv[2], argv[3]);
	}
	else {
		printf("Invalid usage.\n");
	}
}

